@@ -8,7 +8,7 @@ from functools import wraps
 
 
 __all__ = ['Dispatcher', 'command', 'dispatch']
-__version__ = '3.3.1'
+__version__ = '3.4'
 __author__ = 'Alexander Solovyov'
 __email__ = 'alexander@solovyov.net'
 
@@ -16,18 +16,21 @@ __email__ = 'alexander@solovyov.net'
 try:
     import locale
     ENCODING = locale.getpreferredencoding()
-    if not ENCODING or ENCODING == 'mac-roman' or 'ascii' in ENCODING.lower():
+    if (not ENCODING or ENCODING == 'mac-roman' or 'ascii' in ENCODING.lower()
+        or 'ansi' in ENCODING.lower()):
         ENCODING = 'UTF-8'
 except locale.Error:
     ENCODING = 'UTF-8'
 
+
 def write(text, out=None):
     '''Write output to a given stream (stdout by default)'''
     out = out or sys.stdout
-    if isinstance(text, unicode):
-        return out.write(text.encode(ENCODING))
+    if sys.version_info < (3, 0) and isinstance(text, unicode):
+        text = text.encode(ENCODING)
     out.write(text)
 
+
 def err(text):
     '''Write output to stderr'''
     write(text, out=sys.stderr)
@@ -118,13 +121,14 @@ class Dispatcher(object):
             except TypeError:
                 options_ = []
 
-            name_ = name or name_from_python(func.__name__)
+            cmdname = name or name_from_python(func.__name__)
+            scriptname = name or sysname()
             if usage is None:
                 usage_ = guess_usage(func, options_)
             else:
                 usage_ = usage
             prefix = hide and '~' or (shortlist and '^' or '')
-            cmdname = prefix + name_
+            cmdname = prefix + cmdname
             if aliases:
                 cmdname = cmdname + '|' + '|'.join(aliases)
             self._cmdtable[cmdname] = (func, options_, usage_)
@@ -140,7 +144,8 @@ class Dispatcher(object):
                                 None):
                         options_.append(o)
 
-                argv = argv or sys.argv[1:]
+                if argv is None:
+                    argv = sys.argv[1:]
                 try:
                     args, opts = process(argv, options_)
                 except Exception, e:
@@ -152,7 +157,7 @@ class Dispatcher(object):
                     return func.help()
 
                 try:
-                    return call_cmd(name_, func, options_)(*args, **opts)
+                    return call_cmd(scriptname, func, options_)(*args, **opts)
                 except Exception, e:
                     if exchandle(e, func.help):
                         return -1
@@ -175,7 +180,13 @@ class Dispatcher(object):
     def _dispatch(self, args):
         cmd, func, args, options = cmdparse(args, self.cmdtable,
                                             self.globaloptions)
-        args, kwargs = process(args, options)
+        try:
+            args, kwargs = process(args, options)
+        except getopt.GetoptError, e:
+            # FIXME: this is ugly, we set command name here to retrieve it later
+            # in exchandle().
+            e.command = cmd
+            raise
 
         if kwargs.pop('help', False):
             return 'help', self.cmdtable['help'][0], [cmd], {}, options
@@ -212,6 +223,7 @@ class Dispatcher(object):
 
 _dispatcher = None
 
+
 def command(options=None, usage=None, name=None, shortlist=False, hide=False,
             aliases=()):
     global _dispatcher
@@ -221,6 +233,7 @@ def command(options=None, usage=None, name=None, shortlist=False, hide=False,
                                shortlist=shortlist, hide=hide, aliases=aliases)
 command.__doc__ = Dispatcher.command.__doc__
 
+
 def dispatch(args=None, cmdtable=None, globaloptions=None, middleware=None):
     global _dispatcher
     if not _dispatcher:
@@ -287,6 +300,7 @@ def help_(cmdtable, globalopts):
                         aliases[1:])
     return help_inner
 
+
 def help_cmd(func, usage, options, aliases):
     '''show help for given command
 
@@ -329,6 +343,7 @@ def help_cmd(func, usage, options, aliases):
     if options:
         write(''.join(help_options(options)))
 
+
 def help_options(options):
     '''Generator for help on options
     '''
@@ -392,7 +407,7 @@ def process(args, options, preparse=False):
         # copy defaults to state
         if isinstance(default, (list, dict)):
             state[pyname] = copy.copy(default)
-        elif hasattr(default, '__call__'):
+        elif isinstance(default, types.FunctionType):
             funlist.append(pyname)
             state[pyname] = None
         else:
@@ -434,11 +449,18 @@ def process(args, options, preparse=False):
                 k, v = val.split('=')
             except ValueError:
                 raise getopt.GetoptError(
-                    "wrong definition: '%s' (should be in format KEY=VALUE)"
+                    "wrong definition: %r (should be in format KEY=VALUE)"
                     % val)
             state[name][k] = v
         elif t in (types.NoneType, types.BooleanType):
             state[name] = not defmap[name]
+        elif t in (int, float):
+            try:
+                state[name] = t(val)
+            except ValueError:
+                raise getopt.GetoptError(
+                    'invalid option value %r for option %r'
+                    % (val, name))
         else:
             state[name] = t(val)
 
@@ -472,10 +494,12 @@ def cmdparse(args, cmdtable, globalopts):
     possibleopts.extend(globalopts)
     return cmd, cmd and info[0] or None, args, possibleopts
 
+
 def aliases_(cmdtable_key):
     '''Get aliases from a command table key'''
     return cmdtable_key.lstrip("^~").split("|")
 
+
 def findpossible(cmd, table):
     """
     Return cmd -> (aliases, command table entry)
@@ -497,6 +521,7 @@ def findpossible(cmd, table):
 
     return choice
 
+
 def findcmd(cmd, table):
     """Return (aliases, command table entry) for command string."""
     choice = findpossible(cmd, table)
@@ -514,6 +539,7 @@ def findcmd(cmd, table):
 
     raise UnknownCommand(cmd)
 
+
 # --------
 # Helpers
 # --------
@@ -536,6 +562,7 @@ def guess_options(func):
         completer = option[3] if len(option) > 3 else None
         yield (sname, name_from_python(name), default, hlp, completer)
 
+
 def guess_usage(func, options):
     '''Get usage definition for a function
     '''
@@ -555,6 +582,7 @@ def guess_usage(func, options):
         usage.append('[%s ...]' % arginfo.varargs.upper())
     return ' '.join(usage)
 
+
 def exchandle(e, help_func):
     '''Handle internal exceptions and print human-readable information on them
 
@@ -570,18 +598,19 @@ def exchandle(e, help_func):
         help_func(e.args[0])
     elif isinstance(e, getopt.GetoptError):
         err('error: %s\n\n' % e)
-        help_func()
+        # we may get command name here, if we're in multicommand context
+        help_func(getattr(e, 'command', None))
     elif isinstance(e, OpsterError):
         err('%s\n' % e)
     else:
         return False
     return True
 
+
 def call_cmd(name, func, opts, middleware=None):
     '''Wrapper for command call, catching situation with insufficient arguments
-
-    ``depth`` is necessary when there is a middleware in setup
     '''
+    # depth is necessary when there is a middleware in setup
     arginfo = inspect.getargspec(func)
     if middleware:
         tocall = middleware(func)
@@ -598,12 +627,12 @@ def call_cmd(name, func, opts, middleware=None):
         start = None
         if arginfo.varargs and len(args) > (len(arginfo.args) - len(kwargs)):
             for o in opts:
-                optname = o[1]
+                optname = o[1].replace('-', '_')
                 if optname in arginfo.args:
                     if start is None:
                         start = arginfo.args.index(optname)
                     prepend.append(optname)
-            if start is not None: # do we have to prepend anything
+            if start is not None:  # do we have to prepend anything
                 args = (args[:start] +
                         tuple(kwargs.pop(x) for x in prepend) +
                         args[start:])
@@ -616,6 +645,7 @@ def call_cmd(name, func, opts, middleware=None):
             raise
     return inner
 
+
 def call_cmd_regular(func, opts):
     '''Wrapper for command for handling function calls from Python
     '''
@@ -634,12 +664,14 @@ def call_cmd_regular(func, opts):
         return func(*args, **funckwargs)
     return inner
 
+
 def replace_name(usage, name):
     '''Replace name placeholder with a command name'''
     if '%name' in usage:
         return usage.replace('%name', name, 1)
     return name + ' ' + usage
 
+
 def sysname():
     '''Returns name of executing file'''
     name = sys.argv[0]
@@ -649,6 +681,7 @@ def sysname():
         return name[2:]
     return name
 
+
 def pretty_doc_string(item):
     "Doc string with adjusted indentation level of the 2nd line and beyond."
     raw_doc = item.__doc__ or '(no help text available)'
@@ -658,17 +691,20 @@ def pretty_doc_string(item):
     indent = len(lines[1]) - len(lines[1].lstrip())
     return '\n'.join([lines[0]] + map(lambda l: l[indent:], lines[1:]))
 
+
 def name_from_python(name):
     if name.endswith('_') and keyword.iskeyword(name[:-1]):
         name = name[:-1]
     return name.replace('_', '-')
 
+
 def name_to_python(name):
     name = name.replace('-', '_')
     if keyword.iskeyword(name):
         return name + '_'
     return name
 
+
 # --------
 # Autocomplete system
 # --------
@@ -721,6 +757,7 @@ def autocomplete(cmdtable, args, middleware):
 
     sys.exit(1)
 
+
 COMPLETIONS = {
     'bash':
         """
@@ -750,6 +787,7 @@ compctl -K _opster_completion %s
 """
     }
 
+
 @command(name='_completion', hide=True)
 def completion(type=('t', 'bash', 'Completion type (bash or zsh)'),
                # kwargs will catch every global option, which we get
@@ -760,23 +798,27 @@ def completion(type=('t', 'bash', 'Completion type (bash or zsh)'),
     prog_name = os.path.split(sys.argv[0])[1]
     print COMPLETIONS[type].strip() % prog_name
 
+
 # --------
 # Exceptions
 # --------
 
-# Command exceptions
 class OpsterError(Exception):
     'Base opster exception'
 
+
 class AmbiguousCommand(OpsterError):
     'Raised if command is ambiguous'
 
+
 class UnknownCommand(OpsterError):
     'Raised if command is unknown'
 
+
 class ParseError(OpsterError):
     'Raised on error in command line parsing'
 
+
 if __name__ == '__main__':
     import doctest
     doctest.testmod()
